# -*- coding: utf-8 -*-
import datetime
import logging
from requests.exceptions import ConnectionError

from ripple_api.models import Transaction
from ripple_api.ripple_api import account_tx, RippleApiError

from django.conf import settings
from django.db.models import Max


PROCESS_TRANSACTIONS_LIMIT = 200
PROCESS_TRANSACTIONS_TIMEOUT = 270
DEFAULT_MIN_LEDGER_INDEX = getattr(
    settings, 'RIPPLE_TRANSACTION_MONITOR_MIN_LEDGER_INDEX', -1
)

logger = logging.getLogger('ripple')
logger.setLevel(logging.ERROR)


def _get_min_ledger_index(account):
    """
    Gets min leger index for transactions of `account` stored
    in database.
    """
    min_ledger_index = Transaction.objects.filter(
        destination=account,
        status__in=[
            Transaction.RECEIVED, Transaction.PROCESSED,
            Transaction.MUST_BE_RETURN, Transaction.RETURNING,
            Transaction.RETURNED
        ]
    ).aggregate(Max('ledger_index'))
    min_ledger_index = min_ledger_index['ledger_index__max'] or -1

    return max(min_ledger_index, DEFAULT_MIN_LEDGER_INDEX)


def _store_transaction(account, transaction):
    """
    Stores transaction for `account` into database.
    """
    tr_tx = transaction['tx']
    meta = transaction.get('meta', {})

    if meta.get('TransactionResult') != 'tesSUCCESS':
        return

    amount = meta.get('delivered_amount') or tr_tx.get('Amount', {})

    is_unprocessed = (
        tr_tx['TransactionType'] == 'Payment' and
        tr_tx['Destination'] == account and
        isinstance(amount, dict) and
        not Transaction.objects.filter(hash=tr_tx['hash'])
    )
    if is_unprocessed:
        logger.info(
            format_log_message(
                'Saving transaction: %s', transaction
            )
        )

        transaction_object = Transaction.objects.create(
            account=tr_tx['Account'],
            hash=tr_tx['hash'],
            destination=account,
            ledger_index=tr_tx['ledger_index'],
            destination_tag=tr_tx.get('DestinationTag'),
            source_tag=tr_tx.get('SourceTag'),
            status=Transaction.RECEIVED,
            currency=amount['currency'],
            issuer=amount['issuer'],
            value=amount['value']
        )

        logger.info(
            format_log_message(
                "Transaction saved: %s", transaction_object
            )
        )


def format_log_message(message, transaction=None, *args):
    """
    Message log formatter for processors.
    """
    if transaction or args:
        format_args = [transaction]
        format_args.extend(args)
        return message % tuple(format_args)
    else:
        return message


def monitor_transactions(account):
    """
    Gets new transactions for `account` and store them in DB.
    """
    start_time = datetime.datetime.now()
    logger.info(
        format_log_message(
            'Looking for new ripple transactions since last run'
        )
    )
    ledger_min_index = _get_min_ledger_index(account)
    marker = None
    has_results = True

    try:
        timeout = settings.RIPPLE_TIMEOUT
    except AttributeError:
        timeout = 5

    while has_results:
        try:
            response = account_tx(account,
                                  ledger_min_index,
                                  limit=PROCESS_TRANSACTIONS_LIMIT,
                                  marker=marker,
                                  timeout=timeout)
        except (RippleApiError, ConnectionError), e:
            logger.error(format_log_message(e))
            break

        transactions = response['transactions']
        marker = response.get('marker')
        has_results = bool(marker)

        for transaction in transactions:
            _store_transaction(account, transaction)

        transactions_timeout_reached = (
            datetime.datetime.now() - start_time >= datetime.timedelta(
                seconds=PROCESS_TRANSACTIONS_TIMEOUT
            )
        )

        if transactions_timeout_reached and has_results:
            has_results = False
            logger.error(
                'Process_transactions command terminated because '
                '(%s seconds) timeout: %s',
                PROCESS_TRANSACTIONS_TIMEOUT, unicode(marker)
            )



# -*- coding: utf-8 -*-
import json

from django.conf import settings
from django.test import TestCase

from requests import Response

from mock import patch
from decimal import Decimal

import ripple_api

sequence = 12
destination_account = u'rJobmmpNqozqY7MzwGkRs1VLEBJ7H5Pjrp'
wrong_secret = "somesecret"

responses = {
    "account_lines": {
        u'status': u'success',
        u'lines': [
            {
                u'account': u'rhhPzptf4EdiRRopSVC3AEbHwDa9df8y2i',
                u'quality_out': 0,
                u'quality_in': 0,
                u'currency': u'USD',
                u'limit_peer': u'5',
                u'limit': u'2.5',
                u'balance': u'1.759565201742073',
            }, {
                u'account': u'rhhPzptf4EdiRRopSVC3AEbHwDa9df8y2i',
                u'quality_out': 0,
                u'no_ripple_peer': True,
                u'quality_in': 0,
                u'currency': u'CCK',
                u'limit_peer': u'2',
                u'limit': u'0',
                u'balance': u'0',
            }, {
                u'account': u'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7',
                u'quality_out': 0,
                u'quality_in': 0,
                u'currency': u'CCK',
                u'limit_peer': u'0',
                u'limit': u'1000000',
                u'balance': u'7.51418646934461',
            }, {
                u'account': u'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7',
                u'quality_out': 0,
                u'quality_in': 0,
                u'currency': u'USD',
                u'limit_peer': u'0',
                u'limit': u'10000',
                u'balance': u'0.7907',
            }],
        u'account': u'rJmEYVEB7Bw16Kt9FYHK74Xrgery6gN6AB',
        u'validated': False,
        u'ledger_current_index': 12159817,
    },
    "account_info": {
        u'status': u'success',
        u'validated': True,
        u'account_data': {
            u'OwnerCount': 3,
            u'index': u'BD2C319063E47D3546796E19DE5716B14107514C4AF61C5FE0C528637D17DA20',
            u'Account': u'rJmEYVEB7Bw16Kt9FYHK74Xrgery6gN6AB',
            u'PreviousTxnID': u'847AA9D8B49955D1216BAA9D3262AFC3DAB5CCB9FD9ED12A0F5FA65B874D4E2B',
            u'Sequence': 44,
            u'LedgerEntryType': u'AccountRoot',
            u'Flags': 0,
            u'PreviousTxnLgrSeq': 12153785,
            u'Balance': u'50488267',
            },
        u'ledger_index': 12159865,
    }
}


def side_effect(data, servers=None, server_url=None, api_user=None,
                api_password=None, timeout=5):
    method = data["method"]
    if method in responses:
        return responses[method]

    raise ripple_api.RippleApiError(
        'Bad request', '',
        'Use only account_lines or account_info methods')


class BalanceTestCase(TestCase):

    def setUp(self):
        self.servers = settings.RIPPLE_API_DATA

    @patch('ripple_api.ripple_api.call_api')
    def test_balance_xrp(self, balance_mock):
        """ Test if balance calls =account_info= rippled-JSON-RPC method
            to obtain XRP Balance
        """

        balance_mock.side_effect = side_effect

        xrp_balance = ripple_api.balance(settings.RIPPLE_ACCOUNT,
                                         issuers=None, currency="XRP",
                                         servers=self.servers)

        self.assertEqual(xrp_balance, Decimal("50.488267"))

    @patch('ripple_api.ripple_api.call_api')
    def test_balance_non_xrp(self, call_api_mock):
        """ Testing if balance calls =account_lies= rippled-JSON-XRP method
            to obtain balances in non XRP currencies
        """

        call_api_mock.side_effect = side_effect
        
        usd_balance = ripple_api.balance(settings.RIPPLE_ACCOUNT,
                                         issuers=None, currency="USD",
                                         servers=self.servers)
        self.assertEqual(usd_balance, Decimal("2.550265201742073"))

        cck_balance = ripple_api.balance(settings.RIPPLE_ACCOUNT,
                                         issuers=None, currency="CCK",
                                         servers=self.servers)
        self.assertEqual(cck_balance, Decimal("7.51418646934461"))



# -*- coding: utf-8 -*-
import logging

from django.conf import settings
from django.core.management import BaseCommand

from ripple_api.ripple_api import tx, RippleApiError
from ripple_api.models import Transaction
from ripple_api.tasks import sign_task, submit_task
from ripple_api.management.transaction_processors import (
    monitor_transactions,
    format_log_message
)


MAX_RESULTS = 200

logger = logging.getLogger('ripple')
logger.setLevel(logging.ERROR)


class Command(BaseCommand):
    help = 'Command that processes transactions.'

    def handle(self, **options):
        self.retry_failed_transactions()
        monitor_transactions(account=settings.RIPPLE_ACCOUNT)
        self.return_funds()
        self.submit_pending_transactions()
        self.check_submitted_transactions()

    def check_submitted_transactions(self):
        """
        Check final disposition of transactions.
        """
        logger.info(
            format_log_message(
                'Checking submitted transactions'
            )
        )

        submitted_transactions = Transaction.objects.filter(
            status=Transaction.SUBMITTED
        )
        for transaction in submitted_transactions:
            try:
                response = tx(transaction.hash)
                logger.info(format_log_message(response))
            except RippleApiError as e:
                logger.error(
                    format_log_message(
                        "Error processing %s: %s", transaction, e
                    )
                )
                if e.error == 'txnNotFound':
                    logger.info(
                        format_log_message(
                            'Setting transaction status to Failed for %s',
                            transaction
                        )
                    )
                    transaction.status = Transaction.FAILURE
                    transaction.save()
                continue

            status = response.get('meta', {}).get('TransactionResult')

            if status == 'tesSUCCESS':
                transaction.status = Transaction.SUCCESS
                transaction.save()

                if transaction.parent:
                    transaction.parent.status = Transaction.RETURNED
                    transaction.parent.save()

                logger.info(format_log_message(
                        "Transaction: %s to %s was complete.",
                        transaction, transaction.destination
                    )
                )
            else:
                logger.info("Transaction status: %s" % status)

    def submit_pending_transactions(self):
        """
        Submit transactions that was signed, but connection error occurred
        when submit it.
        """
        logger.info(format_log_message('Submit pending transactions'))
        pending_transactions = Transaction.objects.filter(
            status=Transaction.PENDING
        )
        for transaction in pending_transactions:
            logger.info(format_log_message('Submit: %s', transaction))
            submit_task.apply((transaction.pk,))

    def return_funds(self):
        logger.info('Returning failed stakes')
        for transaction in Transaction.objects.filter(
                status=Transaction.MUST_BE_RETURN):
            logger.info("Transaction %s must be return." % transaction.pk)

            ret_transaction = Transaction.objects.create(
                account=settings.RIPPLE_ACCOUNT,
                destination=transaction.account,
                currency=transaction.currency,
                value=transaction.value,
                status=Transaction.PENDING,
                parent=transaction
            )
            sign_task.apply((ret_transaction.pk, settings.RIPPLE_SECRET))
            transaction.status = Transaction.RETURNING
            transaction.save()
            logger.info(
                "New transaction created for returning %s", ret_transaction.pk
            )

    def retry_failed_transactions(self):
        logger.info('Retrying failed transactions')
        failed_transactions = Transaction.objects.filter(
            status=Transaction.FAILURE
        )
        for transaction in failed_transactions:
            logger.info(format_log_message('Found %s', transaction))

            retry_transaction = Transaction.objects.create(
                account=transaction.account,
                destination=transaction.destination,
                currency=transaction.currency,
                value=transaction.value,
                status=Transaction.PENDING,
                parent=transaction.parent
            )
            sign_task.apply((retry_transaction.pk, settings.RIPPLE_SECRET))
            logger.info(
                "New transaction created for returning %s",
                retry_transaction.pk
            )
            transaction.status = Transaction.FAIL_FIXED
            transaction.save()
            logger.info("Fixed the transaction")



# -*- coding: utf-8 -*-
import json

from django.conf import settings
from django.test import TestCase

from requests import Response

from mock import patch

from .ripple_api import trust_set, RippleApiError

sequence = 12
destination_account = u'rJobmmpNqozqY7MzwGkRs1VLEBJ7H5Pjrp'
wrong_secret = "somesecret"

data = {
    u"engine_result": u"tesSUCCESS",
    u"engine_result_code": 0,
    u"engine_result_message": (u"The transaction was applied. Only final " +
                               u"in a validated ledger."),
    u"status": u"success",
    u"tx_blob": u"-- hexBinary data --",
    u"tx_json": {
        u"Account": settings.RIPPLE_ACCOUNT,
        u"Fee": u"10000",
        u"Flags": 262144,
        u"LimitAmount": {
            u"currency": u"USD",
            u"issuer": destination_account,
            u"value": u"1"
        },
        u"Sequence": sequence,
        u"SigningPubKey": u"-- hexBinary data of SigningPubKey --",
        u"TransactionType": u"TrustSet",
        u"TxnSignature": u"-- hexBinary data of TxnSignature --",
        u"hash": u"-- hexBinary data of hash --"
    }
}

error_data = {
    u"error": "badSecret",
    u"error_code": 39,
    u"error_message": "Secret does not match account.",
    u"request": {
        u"command": u"submit",
        u"secret": wrong_secret,
        u"tx_json": {
            u"Account": settings.RIPPLE_ACCOUNT,
            u"Fee": "10000",
            u"Flags": 262144,
            u"LimitAmount": {
                u"currency": "USD",
                u"issuer": destination_account,
                u"value": "1"
            },
            u"TransactionType": "TrustSet"
        }
    },
    u"status": "error"
}


class TrustSetTestCase(TestCase):

    def setUp(self):
        pass

    @patch('requests.post')
    def test_trust_set_error(self, post_mock):
        """Test if RippleApiError raised in case when secret is wrong"""
        response = Response()
        response._content = json.dumps({u"result": error_data})
        post_mock.return_value = response

        exp_msg = u'39: badSecret. Secret does not match account.'
        with self.assertRaisesMessage(RippleApiError, exp_msg):
            trust_set(
                settings.RIPPLE_ACCOUNT, wrong_secret,
                destination_account,
                1, u"USD", flags={"AllowRipple": False, "Freeze": True}
            )

    @patch('requests.post')
    def test_trust_set_success(self, post_mock):
        response = Response()
        response._content = json.dumps({u"result": data})
        post_mock.return_value = response

        result = trust_set(
            settings.RIPPLE_ACCOUNT, settings.RIPPLE_SECRET,
            destination_account,
            1, u"USD", flags={"AllowRipple": True}
        )

        self.assertDictEqual(result, data)



# -*- coding: utf-8 -*-
import unittest

from decimal import Decimal
from django.conf import settings
from django.test import TestCase

from mock import patch

from .trade import sell_all


part_pay = 0.5
part_get = 0.1
amount_get = 1
amount_pay = 2
sequence = 1
balance_pay = 100
balance_get = 200

AffectedNodes = {
    u'DeletedNode': {
        'LedgerEntryType': 'Offer',
        'PreviousFields': {
            u'TakerPays': {
                u'currency': 'TUS',
                u'value': str(part_pay),
                u'issuer': u'rJobmmpNqozqY7MzwGkRs1VLEBJ7H5Pjrp'
            },
            u'Account': u'rJobmmpNqozqY7MzwGkRs1VLEBJ7H5Pjrp',
            u'TakerGets': {
                u'currency': 'TBT',
                u'value': str(part_get),
                u'issuer': u'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
            },
        }
    },
}
data = {
    u'status': u'success',
    u'engine_result': u'tesSUCCESS',
    u'engine_result_message': u'The transaction was applied.',
    u'engine_result_code': 0,
    u'tx_blob': u'test',
    u'tx_json': {
        u'Account': settings.RIPPLE_ACCOUNT,
        u'hash': u'test_hash',
        u'Sequence': sequence,
        u'TakerPays': {
            u'currency': 'TBT',
            u'value': amount_pay,
            u'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
        },
        u'TakerGets': {
            u'currency': 'TUS',
            u'value': amount_get,
            u'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7',
            u'TransactionType': u'OfferCreate'
        }
    }
}

tx_data = {
    u'status': u'success',
    u'Account': settings.RIPPLE_ACCOUNT,
    u'Sequence': sequence,
    u'TakerPays': {
        u'currency': 'TBT',
        u'value': str(amount_pay),
        u'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
    },
    u'meta': {
        u'TransactionResult': u'tesSUCCESS',
        u'AffectedNodes': [{
            u'ModifiedNode': {
                u'LedgerEntryType': u'Offer',
                u'FinalFields': {
                    u'Account': u'rfaqM2Mkc9UT2RAWvLFfUivUqhyH4i5qgd',
                    u'Sequence': sequence,
                    u'TakerPays': {
                        u'currency': 'TUS',
                        u'value': str(amount_pay),
                        u'issuer': u'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
                    },
                    u'TakerGets': {
                        u'currency': 'TBT',
                        u'value': str(amount_get),
                        u'issuer': u'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
                    },
                    u'Flags': 0
                },
                u'PreviousFields': {
                    u'TakerPays': {
                        u'currency': 'TUS',
                        u'value': str(balance_pay),
                        u'issuer': u'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
                    },
                    u'TakerGets': {
                        u'currency': 'TBT',
                        u'value': str(balance_get),
                        u'issuer': u'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
                    }
                }
            }
        }]
    },
    u'TakerGets': {
        u'currency': 'TUS',
        u'value': str(amount_get),
        u'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7',
    },
    u'validated': True,
    u'TransactionType': u'OfferCreate'
}


def generate_ripple_transaction_meta(final_balance, previous_balance):
    return {
        'AffectedNodes': [
            {
                "ModifiedNode": {
                    "FinalFields": {
                        "Balance": {
                            "currency": "TUS",
                            "issuer": "rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7",
                            "value": str(final_balance)
                        },
                        "HighLimit": {
                            "currency": "TUS",
                            "issuer": 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7',
                            "value": "10000"
                        },
                    },
                    "LedgerEntryType": "RippleState",
                    "PreviousFields": {
                        "Balance": {
                            "currency": "TUS",
                            "issuer": "rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7",
                            "value": str(previous_balance)
                        }
                    },
                }
            },
            {
                "ModifiedNode": {
                    "FinalFields": {
                        "Account": settings.RIPPLE_ACCOUNT,
                        "Balance": "80678117",
                    },
                    "LedgerEntryType": "AccountRoot",
                    "PreviousFields": {
                        "Balance": "80688117",
                        "Sequence": 959
                    },
                }
            },
            {
                "ModifiedNode": {
                    "FinalFields": {
                        "Balance": {
                            "currency": "TUS",
                            "issuer": "rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7",
                            "value": "0.1"
                        },
                        "HighLimit": {
                            "currency": "TUS",
                            "issuer": "rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7",
                            "value": "0"
                        },
                    },
                    "LedgerEntryType": "RippleState",
                    "PreviousFields": {
                        "Balance": {
                            "currency": "TUS",
                            "issuer": "rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7",
                            "value": "0.2"
                        }
                    },
                }
            }
        ],
        'TransactionResult': 'tesSUCCESS'
    }


# FIXME: disable tests for now as these are not consistent with
# the implementation at all
# https://github.com/42cc/ripple_api/issues/24
@unittest.skip
class TradeTestCase(TestCase):
    def setUp(self):
        self.create_data = data
        self.tx_data = tx_data

    @patch('ripple_api.ripple_api.call_api')
    @patch('ripple_api.trade.create_offer')
    def test_offer_create_error(self, create_offer_mock, call_api_mock):
        """Test if do not sell all when has error in offer."""
        self.create_data['engine_result'] = 'error'

        create_offer_mock.return_value = self.create_data
        call_api_mock.return_value = self.create_data

        exchange_result = sell_all(
            sell_needed={
                'value': amount_pay, 'currency': 'TUS',
                'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
            },
            buy_expected={
                'value': amount_get, 'currency': 'TBT',
                'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
            },
        )

        # check result error, sold 0, bought 0
        self.assertEqual(exchange_result['status'], 'error')
        self.assertEqual(exchange_result['status_msg'],
                         'Offer creation failed: error')
        self.assertEqual(exchange_result['sold'], 0)
        self.assertEqual(exchange_result['bought'], 0)

    @patch('ripple_api.ripple_api.call_api')
    @patch('ripple_api.trade.create_offer')
    def test_offer_create_nothing_happened(self, create_offer_mock,
                                           call_api_mock):
        """Test if do not sell all when offer without data 'AffectedNodes'."""
        self.create_data['engine_result'] = 'tesSUCCESS'
        rez = self.tx_data['meta']['AffectedNodes']
        del self.tx_data['meta']['AffectedNodes']

        create_offer_mock.return_value = self.create_data

        call_api_mock.return_value = self.tx_data

        exchange_result = sell_all(
            sell_needed={
                'value': amount_pay, 'currency': 'TUS',
                'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
            },
            buy_expected={
                'value': amount_get, 'currency': 'TBT',
                'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
            },
        )
        self.tx_data['meta']['AffectedNodes'] = rez
        # check result success, sold 0, bought 0
        self.assertEqual(exchange_result['status'], 'error')
        self.assertEqual(exchange_result['status_msg'],
                         "Offer was not identified.")
        self.assertEqual(exchange_result['sold'], 0)
        self.assertEqual(exchange_result['bought'], 0)

    @patch('ripple_api.ripple_api.call_api')
    @patch('ripple_api.trade.create_offer')
    def test_offer_create_not_happened(self, create_offer_mock, call_api_mock):
        """Test if do not sell all when offer without data 'ModifiedNode'."""
        self.create_data['engine_result'] = 'tesSUCCESS'
        rez = self.tx_data['meta']['AffectedNodes'][0]['ModifiedNode']
        del self.tx_data['meta']['AffectedNodes'][0]['ModifiedNode']

        create_offer_mock.return_value = self.create_data

        call_api_mock.return_value = self.tx_data

        exchange_result = sell_all(
            sell_needed={
                'value': amount_pay, 'currency': 'TUS',
                'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
            },
            buy_expected={
                'value': amount_get, 'currency': 'TBT',
                'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
            },
        )
        self.tx_data['meta']['AffectedNodes'][0]['ModifiedNode'] = rez
        # check result success, sold 0, bought 0
        self.assertEqual(exchange_result['status'], 'success')
        self.assertEqual(exchange_result['sold'], 0)
        self.assertEqual(exchange_result['bought'], 0)

    @patch('ripple_api.ripple_api.call_api')
    @patch('ripple_api.trade.create_offer')
    def test_offer_create_sold_a_part(self, create_offer_mock, call_api_mock):
        """Test if correct sell a part, when offer has only part for sell."""
        self.create_data['engine_result'] = 'tesSUCCESS'
        create_offer_mock.return_value = self.create_data

        call_api_mock.return_value = self.tx_data

        exchange_result = sell_all(
            sell_needed={
                'value': amount_pay, 'currency': 'TUS',
                'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
            },
            buy_expected={
                'value': amount_get, 'currency': 'TBT',
                'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
            },
        )

        # check result success, sold only a part
        self.assertEqual(exchange_result['status'], 'success')
        self.assertEqual(exchange_result['bought'],
                         Decimal(balance_get - amount_get))
        self.assertEqual(exchange_result['sold'],
                         Decimal(balance_pay - amount_pay))

    @patch('ripple_api.ripple_api.call_api')
    @patch('ripple_api.trade.create_offer')
    def test_offer_create_sold_everything(self, create_offer_mock,
                                          call_api_mock):
        """Test if correct we can sell all."""

        self.create_data['engine_result'] = 'tesSUCCESS'
        del self.tx_data['meta']['AffectedNodes'][0]['ModifiedNode']

        self.tx_data['meta']['AffectedNodes'][-1] = AffectedNodes

        create_offer_mock.return_value = self.create_data

        call_api_mock.return_value = self.tx_data

        exchange_result = sell_all(
            sell_needed={
                'value': amount_pay, 'currency': 'TUS',
                'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
            },
            buy_expected={
                'value': amount_get, 'currency': 'TBT',
                'issuer': 'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7'
            },
        )

        # check result success, sold a part
        self.assertEqual(exchange_result['status'], 'success')
        self.assertEqual(exchange_result['sold'], Decimal("%.6f" % part_pay))
        self.assertEqual(exchange_result['bought'], Decimal("%.6f" % part_get))



# -*- coding: utf-8 -*-

# system imports:
import json
import logging
import socket
from decimal import Decimal
import ssl

# thirdparty imports:
import requests


logger = logging.getLogger(__name__)

ENGINE_SUCCESS = 'tesSUCCESS'

'''
   Trust lines flags definition
   Docs: https://ripple.com/build/transactions/#trustset
'''
# Authorize the other party to hold issuances from this
# account. (No effect unless using the asfRequireAuth
# AccountSet flag.) Cannot be unset.
#
# For details see:
# https://ripple.com/build/transactions/#accountset-flags
SET_AUTH = 0x00010000        # tfSetAuth = 65536

# Blocks rippling between two trustlines of the same currency,
# if this flag is set on both. (See No Ripple for details.)
#
# For details see:
# https://ripple.com/knowledge_center/understanding-the-noripple-flag/
SET_NORIPPLE = 0x00020000    # tfSetNoRipple = 131072

# Clears the No-Rippling flag. (See No Ripple for details.)
#
# For details see:
# https://ripple.com/knowledge_center/understanding-the-noripple-flag/
CLEAR_NORIPPLE = 0x00040000   # tfClearNoRipple = 262144

# Freeze the trustline.
#
# For details see:
# https://wiki.ripple.com/Freeze
SET_FREEZE = 0x00100000      # tfSetFreeze = 1048576

# Unfreeze the trustline
#
# For details see:
# https://wiki.ripple.com/Freeze
CLEAR_FREEZE = 0x00200000    # tfClearFreeze = 2097152

# No flags set
NO_FLAGS = 0x0000000
'''
    end of Trust lines flags definition
'''


class RippleApiError(Exception):

    def __init__(self, error, code, message):
        self.error = error
        self.code = code
        self.message = message

    def __str__(self):
        return '%s: %s. %s' % (self.code, self.error, self.message)


def call_api(data, servers=None, server_url=None, api_user=None,
             api_password=None, timeout=5):
    try:
        from django.conf import settings
        if server_url and not (api_user or api_password):
            servers = filter(
                lambda item: item.get('RIPPLE_API_URL', '') == server_url,
                settings.RIPPLE_API_DATA
            )
            servers = servers or [{'RIPPLE_API_URL': server_url}]
        elif server_url and (api_user or api_password):
            servers = [
                {
                    'RIPPLE_API_URL': server_url,
                    'RIPPLE_API_USER': api_user,
                    'RIPPLE_API_PASSWORD': api_password,
                }
            ]
        else:
            from django.core.exceptions import ImproperlyConfigured
            # we have django in virtual env, but not necessarily
            # a settings.RIPPLE_API_DATA
            try:
                servers = settings.RIPPLE_API_DATA
            except ImproperlyConfigured:
                raise ImportError
    except ImportError:
        if servers is None:
            if server_url:
                servers = [
                    {
                        'RIPPLE_API_URL': server_url,
                        'RIPPLE_API_USER': api_user,
                        'RIPPLE_API_PASSWORD': api_password,
                        }
                    ]
            else:
                raise RippleApiError(
                    'Config', '',
                    'Either use django settings or send servers explicitly')

    error = None
    timeouts = 0

    for server_config in servers:
        url = server_config.get('RIPPLE_API_URL', '')
        user = server_config.get('RIPPLE_API_USER', '')
        pwd = server_config.get('RIPPLE_API_PASSWORD', '')
        auth = (user, pwd) if user or pwd else None
        try:
            response = requests.post(
                url, json.dumps(data), auth=auth, verify=False, timeout=timeout)
        except TypeError:  # e.g. json encode error
            raise
        except (requests.exceptions.Timeout, ssl.SSLError, socket.timeout) as e:
            timeouts += 1
            continue
        except Exception as e:
            error = e
            continue

        try:
            result = response.json()['result']
        except ValueError:
            error = RippleApiError('Error', '', response.text)
            continue

        if not 'error' in result:
            return result
        else:
            error = RippleApiError(
                result['error'],
                result.get('error_code', 'no_code'),
                result.get('error_message', 'no_message'),
            )
            continue

    if timeouts == len(servers):
        raise RippleApiError('Timeout', '', 'rippled timed out')

    raise error


def account_info(account, servers=None, server_url=None, api_user=None,
                 api_password=None, timeout=5):

    request = {
        "method": "account_info",
        "params": [
            {
                "account": account,
                "strict": True,
                "ledger_index": "validated"
            }
        ]
    }

    return call_api(request, servers=servers, server_url=server_url,
                    api_user=api_user, api_password=api_password,
                    timeout=timeout)


def account_tx(
        account, ledger_index_min=-1, ledger_index_max=-1, binary=False,
        forward=False, limit=None, marker=None,
        server_url=None, api_user=None, api_password=None, timeout=5):
    """
    Fetch a list of transactions that applied to this account.

    Params:
        `account`:
            Ripple account.

        `ledger_index_min` or `ledger_index_max` of -1 indicates the first and last fully-validated ledgers the server
            has available. This range may not be stable. If you've gotten a count this way and you wish to query the
            same set, make sure to use the returned ledger_index_min and ledger_index_max on future queries.

        `binary`:
            True, to return transactions in hex rather than JSON.

        `forward`:
            True, to sort in ascending ledger order.

        `limit`:
            Maximum number of results to provide.

        `marker`:
            The point to resume from.

    """
    data = {"method": "account_tx",
            "params": [{
                "account": account,
                "ledger_index_min": ledger_index_min,
                "ledger_index_max": ledger_index_max,
                "binary": binary,
                "forward": forward}]}
    if limit:
        data['params'][0]['limit'] = limit
    if marker:
        data['params'][0]['marker'] = marker

    return call_api(data, server_url, api_user, api_password, timeout=timeout)


def tx(transaction_id, servers=None, server_url=None, api_user=None,
       api_password=None, timeout=5):
    """
    Return information about a transaction.

    Params:

        `transaction_id`:
            Hash of transaction.
    """
    data = {"method": "tx",
            "params": [{'transaction': transaction_id}]}

    return call_api(data, servers=servers, server_url=server_url,
                    api_user=api_user, api_password=api_password,
                    timeout=timeout)


def path_find(account, destination, amount, source_currencies=None, servers=None,
              server_url=None, api_user=None, api_password=None, timeout=5):
    '''
    Before sending IOU you need to find paths to the destination account

    Params:
        `account`:
            Source account

        `destination`:
            Destination account

        `amount`:
            IOU amount as in https://ripple.com/wiki/RPC_API#path_find

        `source_currencies`:
            List of source IOU currencies you'd like to pay with
    '''
    data = {'method': 'ripple_path_find',
            'params': [{
                'command': 'ripple_path_find',
                'source_account': account,
                'destination_account': destination,
                'destination_amount': amount,
                }]
        }
    if source_currencies:
       data['params'][0]['source_currencies'] = source_currencies
    return call_api(data, servers=servers, server_url=server_url,
                    api_user=api_user, api_password=api_password,
                    timeout=timeout)


def sign(account, secret, destination, amount, send_max=None, paths=None,
         flags=None, destination_tag=None, transaction_type='Payment',
         servers=None, server_url=None, api_user=None, api_password=None,
         timeout=5, fee=10000):
    """
    After you've created a transaction it must be cryptographically signed using the secret belonging to the owner of
    the sending address. Signing a transaction prior to submission allows you to maintain closer control over
    transaction history. It also allows you to resubmit a previous transaction in the case of a connection failure
    without needing to set up another transaction.

    Params:
        `account`:
            Ripple account.

        `secret`:
            Secret key of sender.

        `destination`:
            The receiving account.

        `amount`:
            The amount and currency for the destination to receive.

            If currency is  XRP, then amount is simply value of payment.
            In other cases, amount have that format:
            {
              "currency" : currency,
              "value" : string,
              "issuer" : account_id,
            }

            To deliver a specific issuer's currency, set the issuer to the account of the issuer.
            To not specify a specific issuer, set the issuer to the receiving account.

        `destination_tag`:
            Tag to identify the reason for payment.
    """
    data = {
        "method": "sign",
        "params": [
            {
                "secret": secret,
                "tx_json":
                {
                    "TransactionType": transaction_type,
                    "Account": account,
                    "Destination": destination,
                    "Amount": amount,
                    "Fee": fee,
                }
            }]}

    if send_max:
        data['params'][0]['tx_json']['SendMax'] = send_max
    if paths:
        data['params'][0]['tx_json']['Paths'] = paths
    if flags:
        data['params'][0]['tx_json']['Flags'] = flags
    if destination_tag:
        data['params'][0]['tx_json']['DestinationTag'] = destination_tag

    return call_api(data, servers=servers, server_url=server_url,
                    api_user=api_user, api_password=api_password,
                    timeout=timeout)


def submit(tx_blob, fail_hard=False, servers=None, server_url=None,
           api_user=None, api_password=None, timeout=5):
    """
    Submits a transaction to the network.

    Params:
        `tx_blob`:
            It  is the signed, encrypted transaction request generated by call of sign is represented as a very long
             string of hexadecimal digits(several hundred characters in length).

    """
    data = {
        "method": "submit",
        "params": [{
            "tx_blob": tx_blob,
            'fail_hard': fail_hard,
            }]}

    return call_api(data, servers=servers, server_url=server_url,
                    api_user=api_user, api_password=api_password,
                    timeout=timeout)


def balance(account, issuers, currency, servers=None, server_url=None,
            api_user=None, api_password=None, timeout=5):

    if currency == "XRP":
        info = account_info(account, servers=servers, server_url=server_url,
                            api_user=api_user, api_password=api_password,
                            timeout=timeout)
        return Decimal(info["account_data"]["Balance"]) / Decimal(1e6)

    results = call_api({'method': 'account_lines',
                        'params': [{'account': account}]
                        },
                       servers=servers,
                       server_url=server_url,
                       api_user=api_user,
                       api_password=api_password,
                       timeout=timeout,
                       )
    total = Decimal('0.0')
    for line in results['lines']:
        if line['currency'] == currency:
            if issuers is None or line['account'] in issuers:
                total += Decimal(line['balance'])
    return total


def is_trust_set(trusts, peer, currency='', limit=0,
                 servers=None, server_url=None, api_user=None,
                 api_password=None, timeout=5):
    """
    checks if 'trusts' trusts 'peer' with specified currency and limit


    Params:
        `trusts`:
            ripple address, that trusts or not 'peer'
        `peer`:
            ripple_address, that is trusted by  by 'trusts'
        `currency` (optional):
            currency in which trust should be verified
        `limit` (optional):
            minimal amount of trust

    Returns boolean

    """
    trust_result = False

    trust_lines = call_api(
        {
            'method': 'account_lines',
            'params': [{
                'account': trusts,
                'peer': peer
            }]
        },
        servers=servers, server_url=server_url,
        api_user=api_user, api_password=api_password,
        timeout=timeout,
    )

    status = trust_lines['status'] == 'success'
    trusts = trust_lines['lines']
    if status and trusts and not currency:
        trust_result = True

    elif status and trusts and currency:
        for trust in trusts:

            if currency == trust['currency']:
                trust_result = float(limit) <= float(trust['limit'])
                break

    return trust_result


def book_offer(
    taker_pays_curr, taker_pays_curr_issuer, taker_gets_curr, taker_gets_curr_issuer, taker_address='',
    ledger='current', marker='', autobridge=True, server_url=None,
    api_user=None, api_password=None, timeout=5):
    """
    Gets currency exchange rates

    Params:
        'taker_pays':
            Specified in the following forms 'XRP' or 'USD/rHb9CJAWyB4rj91VRWn96DkukG4bwdtyTh'
            The currency and issuer the taker pays.
            Do not specify an issuing account if the currency is XRP.
        'taker_gets':
            Specified in the following forms 'XRP' or 'USD/rHb9CJAWyB4rj91VRWn96DkukG4bwdtyTh'
            The currency and issuer the taker pays. Do not specify an issuing
            account if the currency is XRP.
        'ledger' (optional):
            "current" (default). "closed", "validated", ledger_index, or ledger.
        'taker' (optional):
            The address of the taker. This affects the funding of offers by
            owners as they may need to pay transfer fees.
            For a neutral point of view specify ADDRESS_ONE (rrrrrrrrrrrrrrrrrrrrBZbvji).
        'marker' (optional):
            Specify the paging marker as JSON. Defaults to "".
            Token indicating start of page, it is returned from a previous invocation.
        'autobridge' (optional):
            If present, specifies synthesize orders through XRP books. Defaults to true
    """
    taker_pays = 'XRP' if taker_pays_curr == 'XRP' else {
        "currency": taker_pays_curr, "issuer": taker_pays_curr_issuer
    }
    taker_gets = 'XRP' if taker_gets_curr == 'XRP' else {
        "currency": taker_gets_curr, "issuer": taker_gets_curr_issuer
    }
    data = {
        "method": "book_offers",
        "params": [{
            "taker_pays": taker_pays,
            "taker_gets": taker_gets,
            "ledger": ledger,
            "marker": marker,
            "autobridge": autobridge}]
        }
    if taker_address:
        data["params"][0]["taker"] = taker_address

    return call_api(data, server_url, api_user, api_password, timeout=timeout)


def create_offer(taker_pays, taker_gets,
                 account=None,
                 secret=None,
                 timeout=5,
                 fee=10000, flags=0):
    """
    taker - user, that accepts your offer
    takes:
        taker_pays -  {
            'value':   - float - amount to buy
            'currency': - str   - currency
            'issuer':   - str   - issuer
        }
        or Decimal(amount) if currency is XRP
        taker_gets -  {
            'value':   - float - amount to sell
            'currency': - str   - currency
            'issuer':   - str   - issuer
        }
        or Decimal(amount) if currency is XRP
    """

    if isinstance(taker_pays, dict):
        taker_pays['value'] = "%.12f" % taker_pays['value']
    else:
        taker_pays = "%.12f" % taker_pays
    if isinstance(taker_gets, dict):
        taker_gets['value'] = "%.12f" % taker_gets['value']
    else:
        taker_gets = "%.12f" % taker_gets
    offer = {
        "method": "submit",
        "params": [{
            "secret": secret,
            "tx_json": {
                "TransactionType": "OfferCreate",
                "Fee": str(fee),
                "Flags": flags,
                "Account": account,
                "TakerPays": taker_pays,
                "TakerGets": taker_gets,
            },
        }]
    }

    return call_api(offer, timeout=timeout)


def convert(amount_from,
            currency_from, issuer_from,
            currency_to, issuer_to,
            taker_address='', offers_info='',
            call_offer=True, default_rate=0,
            sell=False, reverse=False):

    offers_all = []
    if reverse:
        currency_from, currency_to = currency_to, currency_from
        issuer_from, issuer_to = issuer_to, issuer_from

    def check_offers(offers_info):
        status = offers_info['status']
        offers_all = offers_info['offers']
        if not offers_all:
            status = 'no_offers'
        return status, offers_all

    # find offers from provided
    if offers_info:
        status, offers_all = check_offers(offers_info)

    if not offers_all and call_offer:
        try:
            offers_info = book_offer(
                currency_from, issuer_from, currency_to, issuer_to,
                taker_address=taker_address,
            )
            # conn_status = offers_info['status']
            status, offers_all = check_offers(offers_info)
        except Exception:
            status = 'error'

    # convert
    amount_to = 0
    if offers_all:
        convert_left = Decimal(amount_from)
        for offer in offers_all:
            if 'taker_gets_funded' in offer:
                pays = Decimal(extract_value(offer['taker_pays_funded']))
                gets = Decimal(extract_value(offer['taker_gets_funded']))
            else:
                pays = Decimal(extract_value(offer['TakerPays']))
                gets = Decimal(extract_value(offer['TakerGets']))

            if not sell:
                if convert_left < gets:
                    rate = Decimal(offer['quality'])
                    amount_to += convert_left * rate
                    convert_left = 0
                    break
                convert_left -= gets
                amount_to += pays
            else:
                if convert_left < pays:
                    rate = Decimal(offer['quality'])
                    amount_to += convert_left / rate
                    convert_left = 0
                    break
                convert_left -= pays
                amount_to += gets

        status = 'success'
        if convert_left:
            rate = Decimal(offers_all[-1]['quality'])
            amount_to += convert_left * rate if not sell else convert_left / rate
            status = 'attn_not_enough_funds'

    # get default rate if nothing found
    elif not offers_all and default_rate:
        status = 'default' if not status else status
        amount_to = Decimal(default_rate) * Decimal(amount_from)

    return {'status': status,
            'amount_to': Decimal(amount_to)}


def extract_value(taker_pays_or_gets):
    if isinstance(taker_pays_or_gets, dict):
        return taker_pays_or_gets['value']
    return taker_pays_or_gets


def _error(resp):
    return {'status': resp.get('status'),
            'status_msg': resp.get('error_message')}

def buy_xrp(amount, account, secret, servers=None):
    """Trade USD -> XRP.

    - amount: amount of XRP to buy in drops (1000000 = 1 XRP)
    - account: ripple account
    - secret: account's secret
    """
    logger.info("Trying to find paths")
    paths = path_find(account, account, "%s" % amount, [{"currency": "USD"}],
                      servers=servers)
    if paths['status'] != 'success':
        logger.error('Failed to find paths')
        return _error(paths)

    logger.info("Paths found successfully")
    logger.info("Trying to sign transaction")

    if len(paths['alternatives']) == 0:
        msg = u'No path alternatives (probably no USD or not enough offers)'
        paths['status'] = 'error'
        paths['error_message'] = msg
        logger.error(msg)
        return _error(paths)
    send_max = paths['alternatives'][0]['source_amount']
    result = sign(account, secret, account, amount,
                  send_max=send_max,
                  paths=paths['alternatives'][0]['paths_computed'],
                  flags=0, servers=servers)
    if result['status'] != 'success':
        logger.error('Failed to sign the transaction')
        return _error(result)

    logger.info("Transaction was successfully signed")
    logger.info("Trying to submit transaction")

    blob = result['tx_blob']
    result = submit(blob, servers=servers)
    if result['status'] != 'success':
        return {'status': result['status'],
                'status_msg': result['error']}

    logger.info("Transaction was successfully submitted")

    return {'status': 'success',
            'bought': amount,
            'sold': send_max['value']}


def trust_set(account, secret, destination, amount, currency,
              flags=NO_FLAGS, destination_tag=None,
              servers=None, server_url=None, api_user=None, api_password=None,
              timeout=5, fee=10000):
    """
        Creates, updates or deletes trust line from account to destination
        with amount of currency

        Documentation:
        https://ripple.com/build/transactions/#trustset

        takes:

            account -- id of the ripple account trusts

            secret -- the secret of account trusts

            destination -- id of the ripple account must be trust to

            amount -- amount of trust limit. if Amount is 0 trust line will
                      be deleted from account to destination

            currency -- currency of trust line

            fee -- (optional) XRP drops of ripple fee. Default = 10000 drops

            flags -- (optional) integer or dictionary - {
                "Auth":
                    True, # tfSetAuth - equals to increase flags by SET_AUTH
                "AllowRipple":
                    False, # tfSetNoRipple - equals to increase flags
                           # by SET_NORIPPLE
                    True, # tfClearNoRipple - equals to increase
                          # flags by CLEAR_NORIPPLE
                "Freeze":
                    True, # tfSetFreeze - equals to increase flags
                          # by SET_FREEZE
                    False, # tfClearFreeze - equals to increase flags
                           # by CLEAR_FREEZE
            }. Default equals to { } (empty dictionary)

            destination_tag -- (optional) the tag to explain the transaction

            servers -- (optional) the list of servers to be called to submit
                       transaction

        returns: result field form json-response of rippled server

    """
    if isinstance(flags, dict):
        flags = (
            # tfSetAuth
            (SET_AUTH if flags.get("Auth", False) else 0) +

            # tfClearNoRipple
            (CLEAR_NORIPPLE if flags.get("AllowRipple", None) else 0) +

            # tfSetNoRipple
            (SET_NORIPPLE if not flags.get("AllowRipple", True) else 0) +

            # tfSetFreeze
            (SET_FREEZE if flags.get("Freeze", None) else 0) +

            # tfClearFreeze
            (CLEAR_FREEZE if not flags.get("Freeze", True) else 0)
        )

    trustset = {
        "method": "submit",
        "params": [{
            "secret": secret,
            "tx_json": {
                "TransactionType": "TrustSet",
                "Fee": str(fee),
                "Flags": flags,
                "Account": account,
                "LimitAmount": {
                    "currency": currency,
                    "issuer": destination,
                    "value": "%.2f" % amount
                }
            },
        }]
    }

    logger.info("Trying to submit TrustSet")

    result = call_api(trustset,
                      servers=servers, server_url=server_url,
                      api_user=api_user, api_password=api_password,
                      timeout=timeout
                      )

    if result['status'] == 'success':
        logger.info("TrustSet was successfully submitted")

    return result


def simple_trade(account, secret, currency_from, currency_to, amount):
    """
    Exchange currencies. Uses method similar to what ripple client does in
    'Trade -> Simple' mode.

    :param account: account for which exchange happens
    :param secret: account's secret
    :param currency_from: currency to sell
    :param currency_to: currency to buy
    :param amount: amount to sell. Exchange rate determined automatically

    Both currency params are dicts with currency & issuer key.
    """
    amount = "%.12f" % amount
    rate = convert(
        amount,
        currency_from['currency'],
        currency_from['issuer'],
        currency_to['currency'],
        currency_to['issuer'],
        sell=True,
    )
    if rate['status'] != 'success':
        logger.error('Failed to determine exchange rate')
        return _error(rate)

    value = "%.12f" % rate['amount_to']
    to_buy = dict(value=value, **currency_to)
    paths = path_find(
        account,
        account,
        to_buy,
        [currency_from],
    )
    if paths['status'] != 'success':
        logger.error('Failed to find paths')
        return _error(paths)
    if len(paths['alternatives']) == 0:
        msg = u'No path alternatives (probably no USD or no offers)'
        paths['status'] = 'error'
        paths['error_message'] = msg
        logger.error(msg)
        return _error(paths)

    send_max = paths['alternatives'][0]['source_amount']
    result = sign(
        account,
        secret,
        account,
        to_buy,
        send_max=send_max,
        paths=paths['alternatives'][0]['paths_computed'],
    )
    if result['status'] != 'success':
        logger.error('Failed to sign the transaction')
        return _error(result)

    blob = result['tx_blob']
    result = submit(blob, fail_hard=True)
    ok = (result['status'] == 'success'
          and result.get('engine_result') == ENGINE_SUCCESS)
    if ok:
        return {'status': 'success',
                'bought': to_buy,
                'sold': send_max}
    status = (result['status'], result.get('engine_result'))
    return {'status': '%s: %s' % status,
            'status_msg': result.get('error')}



# -*- coding: utf-8 -*-

from decimal import Decimal
import copy
import time
import logging
import os
from distutils.util import strtobool

from ripple_api import call_api, tx

logger = logging.getLogger(__name__)


def sell_all(buy_expected, sell_needed,
             account, secret, timeout=5, fee=10000,
             default_precission=Decimal('0.00000001'),
             servers=None):
    """
    Exchange the entire sell needed amount, even if it means obtaining more
    than the buy expected amount in exchange.

    takes:
    buy_expected -  {
        'value:     - Decimal - min amount to buy
        'currency': - str     - currency
        'issuer':   - str     - issuer
    }
    sell_needed -  {
        'value':    - Decimal - amount to sell
        'currency': - str     - currency
        'issuer':   - str     - issuer
    }
    account            - ripple account
    secret             - ripple secret
    timeout            - trade timeout (default: 5)
    fee                - ripple fee (default: 10000)
    default_precission - compare precission(default: 0.00000001)
    servers            - ripple servers (default: none)

    returns: {
        'status':      - str    - 'success' / 'error',
        'status_msg':  - str    - 'status description'
        'sold':        - float  - the amount sold
        'bought':      - float  - the amount bought
    }

    """
    logger.info('Trading %s %s -> %s %s' %
                (sell_needed['value'], sell_needed['currency'],
                 buy_expected['value'], buy_expected['currency']))
    offer = sell_all_or_cancel(buy_expected, sell_needed, account, secret,
                               timeout=timeout, fee=fee, servers=servers)
    offer_result = get_trade_result(offer, timeout, servers)
    offer_result['sell_amount_left'] = Decimal(
        "%.12f" % float(sell_needed['value'] - offer_result['sold']))

    if offer_result['sold'] >= sell_needed['value'] - default_precission:
        logger.info("Trade fully funded")
    elif offer_result['sold']:
        logger.info("Trade partially happen")
    else:
        logger.info("Trade didn't happen")

    return offer_result


def get_trade_result(created_offer, timeout, servers=None):
    """
    Get created offer result and process it to human readable format.

    takes:
        ripple offer result returned by CreateOffer call

    returns: {
        'status':      - str   - 'success' / 'error',
        'status_msg':  - str   - 'status description'
        'sold':        - float - the amount sold
        'bought':      - float - the amount bought
    }

    """
    # check offer result
    if not created_offer or created_offer['engine_result'] != 'tesSUCCESS':

        status_msg = "Offer creation failed: %s" % \
            created_offer['engine_result'] if created_offer \
            else 'Offer was not created'
        logger.info(status_msg)
        return {'status': 'error',
                'status_msg': status_msg,
                'sold': 0,
                'bought': 0}

    logger.info("Offer created: %s" % created_offer)
    tr_hash = created_offer['tx_json']['hash']
    logger.info("Transaction: %s" % tr_hash)

    # check transaction result
    transaction = get_transaction_result(tr_hash, timeout, servers)

    # if trade didn't happen
    if 'AffectedNodes' not in transaction.get('meta', ''):
        status_msg = "Offer was not identified."
        logger.info("%s AffectedNodes weren't found." % status_msg)
        return {'status': 'error',
                'status_msg': status_msg,
                'sold': 0,
                'bought': 0}

    logger.info("Offer identified: looking for sold, received amount. \n %s" %
                transaction)
    sold, received = get_sold_received(transaction)
    logger.info("Amount sold: %s, Amount received: %s" %
                (sold, received))

    return {'status': 'success',
            'status_msg': 'Trade successfully happened',
            'sold': sold,
            'bought': received}


def get_transaction_result(tr_hash, timeout, servers, attempts=5, wait=2):
    """
    Get created offer hash and look for offer result after it's processing.

    takes:
        tr_hash  - str  - ripple transaction hash
        timeout  - int  - transaction timeout
        servers  - list - ripple servers
        attempts - int  - number of attempts to check if trade happened
        wait     - int  - seconds to wait until next check
    returns:
        transaction result if it's happened or None

    """
    transaction = {}

    # wait for ripple path find
    if not strtobool(os.environ.get("TESTING", "no")):
        time.sleep(wait)

    for i in xrange(attempts):
        transaction = tx(tr_hash, timeout=timeout, servers=servers)
        # check if offer happened
        if 'AffectedNodes' in transaction:
            break
        # wait for ripple path find a little more
        if not strtobool(os.environ.get("TESTING", "no")):
            time.sleep(wait)
    return transaction


def get_sold_received(transaction):
    """
    Calculate amount sold and amount bought from processed offer result.

    takes:
        transaction  -  dict  - ripple tx call result

    returns:
        pays_sum     -  float - amount sold
        gets_sum     -  float - amount bought

    """
    nodes = transaction['meta']['AffectedNodes']
    pays_sum = 0
    gets_sum = 0
    for n in nodes:

        if n.get('DeletedNode', ''):
            played = n['DeletedNode']
            if played.get('LedgerEntryType', '') == 'Offer':
                fields = played['PreviousFields']
                pays_curr, pays_val = restore_taker_values(fields['TakerPays'])
                gets_curr, gets_val = restore_taker_values(fields['TakerGets'])

                pays_sum += pays_val
                gets_sum += gets_val

        if n.get('ModifiedNode', ''):
            played = n['ModifiedNode']
            if played.get('LedgerEntryType', '') == 'Offer':
                prev_fields = played['PreviousFields']
                final_fields = played['FinalFields']
                pf_pays_curr, pf_pays_val = \
                    restore_taker_values(prev_fields['TakerPays'])
                pf_gets_curr, pf_gets_val = \
                    restore_taker_values(prev_fields['TakerGets'])
                ff_pays_currency, ff_pays_value = \
                    restore_taker_values(final_fields['TakerPays'])
                ff_gets_currency, ff_gets_amount = \
                    restore_taker_values(final_fields['TakerGets'])

                pays_sum += pf_pays_val - ff_pays_value
                gets_sum += pf_gets_val - ff_gets_amount

    return pays_sum, gets_sum


def restore_taker_values(taker):
    if isinstance(taker, dict):
        currency = taker['currency']
        value = Decimal(taker['value'])
    return currency, value


def sell_all_or_cancel(taker_pays, taker_gets,
                       account, secret,
                       timeout, fee=10000, servers=None):

    return create_offer(
        taker_pays, taker_gets, account, secret, timeout, fee,
        flags=0x00080000 | 0x00020000, servers=servers)


def create_offer(taker_pays, taker_gets,
                 account, secret,
                 timeout, fee=10000, flags=0, servers=None):
    """
    Create trading offer.

    takes:
        taker_pays -  {
            'value':    - float - amount to buy
            'currency': - str   - currency
            'issuer':   - str   - issuer
        }
        taker_gets -  {
            'value':    - float - amount to sell
            'currency': - str   - currency
            'issuer':   - str   - issuer
        }
        account    - ripple account
        secret     - ripple secret
        timeout    - trade timeout
        fee        - ripple fee (default: 10000)
        flags      - trade flags (default: 0)
        servers    - ripple servers (default: none)

    returns:
        offer

    taker - user, that accepts your offer

    """

    taker_pays = taker_pays.copy()
    taker_gets = taker_gets.copy()
    taker_pays['value'] = "%.12f" % taker_pays['value']
    taker_gets['value'] = "%.12f" % taker_gets['value']
    offer = {
        "method": "submit",
        "params": [{
            "secret": secret,
            "tx_json": {
                "TransactionType": "OfferCreate",
                "Fee": str(fee),
                "Flags": flags,
                "Account": account,
                "TakerPays": taker_pays,
                "TakerGets": taker_gets,
            },
        }]
    }

    # make offer copy and remove secret
    log_offer = copy.deepcopy(offer)
    log_offer['params'][0].pop('secret', None)
    logger.info('Trade offer: %s' % log_offer)
    return call_api(offer, timeout=timeout, servers=servers)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    operations = (
        migrations.CreateModel(
            name='Transaction',
            fields=(
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('account', models.CharField(max_length=100)),
                ('destination', models.CharField(max_length=100)),
                ('hash', models.CharField(blank=True, max_length=100)),
                ('tx_blob', models.TextField(blank=True)),
                ('currency', models.CharField(max_length=3)),
                ('issuer', models.CharField(max_length=100)),
                ('value', models.CharField(max_length=100)),
                ('source_tag', models.IntegerField(blank=True, null=True)),
                ('destination_tag', models.IntegerField(blank=True, null=True)),
                ('ledger_index', models.IntegerField(blank=True, null=True)),
                ('status', models.SmallIntegerField(choices=[(0, 'Transaction received'), (1, 'Transaction was processed'), (2, 'This transaction must be returned to user'), (3, 'Created new transaction for returning'), (4, 'Transaction was returned'), (5, 'Pending to submit'), (6, 'Transaction was submitted'), (7, 'Transaction was failed'), (8, 'Transaction was completed successfully'), (9, 'Transaction was created but not sign'), (10, 'Transaction was processed after successful submit'), (100, 'The failed transaction was fixed by a new retry')], default=0)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='returning_transaction', to='ripple_api.Transaction')),
            ),
        ),
    )



# -*- coding: utf-8 -*-
from hashlib import sha256


def ripple_address_is_valid(address):
    """
    Checks if ripple ``address`` is valid. Return ``address`` or ``False``

    """
    digits58 = 'rpshnaf39wBUDNEGHJKLM4PQRST7VWXYZ2bcdeCg65jkm8oFqi1tuvAxyz'

    def to_bytes(n, length, endianess='big'):
        h = '%x' % n
        s = ('0'*(len(h) % 2) + h).zfill(length*2).decode('hex')
        return s if endianess == 'big' else s[::-1]

    def decode_base58(bc, length):
        n = 0
        for char in bc:
            n = n * 58 + digits58.index(char)
        return to_bytes(n, length, 'big')

    def check_bc(bc):
        try:
            bcbytes = decode_base58(bc, 25)
        except ValueError:
            return False
        return bcbytes[-4:] == sha256(sha256(bcbytes[:-4]).digest()).digest()[:4]

    if check_bc(address):
        return address
    else:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from os.path import join, dirname
from setuptools import setup, find_packages

setup(
    name='django-ripple_api',
    version='0.0.48',
    packages=find_packages(),
    requires=['python (>= 2.7)', 'requests', 'django_model_utils'],
    install_requires=['requests<2.3.0', 'django-model-utils'],
    tests_require=['mock'],
    description='Python wrapper for the Ripple API',
    long_description=open(join(dirname(__file__), 'README.rst')).read(),
    author='42 Coffee Cups',
    author_email='contact@42cc.co',
    url='https://github.com/42cc/ripple_api',
    download_url='https://github.com/42cc/ripple_api/archive/master.zip',
    license='BSD License',
    keywords=['ripple', 'api'],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
    ],
)



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _

from model_utils import ModelTracker

from signals import transaction_status_changed, transaction_failure_send


class Transaction(models.Model):
    RECEIVED = 0
    PROCESSED = 1
    MUST_BE_RETURN = 2
    RETURNING = 3
    RETURNED = 4
    PENDING = 5
    SUBMITTED = 6
    FAILURE = 7
    SUCCESS = 8
    CREATED = 9
    SUCCESS_PROCESSED = 10
    FAIL_FIXED = 100

    STATUS_CHOICES = (
        (RECEIVED, _(u'Transaction received')),
        (PROCESSED, _(u'Transaction was processed')),
        (MUST_BE_RETURN, _(u'This transaction must be returned to user')),
        (RETURNING, _(u'Created new transaction for returning')),
        (RETURNED, _(u'Transaction was returned')),
        (PENDING, _(u'Pending to submit')),
        (SUBMITTED, _(u'Transaction was submitted')),
        (FAILURE, _(u'Transaction was failed')),
        (SUCCESS, _(u'Transaction was completed successfully')),
        (CREATED, _(u'Transaction was created but not sign')),
        (SUCCESS_PROCESSED,
            _(u'Transaction was processed after successful submit')),
        (FAIL_FIXED, _(u'The failed transaction was fixed by a new retry'))
    )

    account = models.CharField(max_length=100)
    destination = models.CharField(max_length=100)
    hash = models.CharField(max_length=100, blank=True)
    tx_blob = models.TextField(blank=True)

    currency = models.CharField(max_length=3)
    issuer = models.CharField(max_length=100)
    value = models.CharField(max_length=100)

    source_tag = models.IntegerField(null=True, blank=True)
    destination_tag = models.IntegerField(null=True, blank=True)
    ledger_index = models.IntegerField(null=True, blank=True)
    status = models.SmallIntegerField(choices=STATUS_CHOICES, default=RECEIVED)

    parent = models.ForeignKey('self', null=True, blank=True,
                               related_name='returning_transaction')
    created = models.DateTimeField(auto_now_add=True)

    status_tracker = ModelTracker(fields=['status'])

    def __unicode__(self):
        return u'[%s] %s. %s %s from %s to %s' % (
            self.pk, self.created, self.value,
            self.currency, self.account, self.destination
        )

    def save(self, *args, **kwargs):
        created = bool(self.pk)
        super(Transaction, self).save(*args, **kwargs)

        if created and self.status_tracker.previous('status') is not None:
            transaction_status_changed.send(
                sender=self.__class__,
                instance=self,
                old_status=self.status_tracker.previous('status')
            )
        if self.status == self.FAILURE:
            transaction_failure_send.send(sender=self.__class__, instance=self)



# -*- coding: utf-8 -*-
from django.contrib import admin

from .models import Transaction


class TransactionAdmin(admin.ModelAdmin):
    list_display = ('created', 'account', 'destination',
                    'currency', 'value', 'ledger_index', 'status')

admin.site.register(Transaction, TransactionAdmin)



# -*- coding: utf-8 -*-
import django.dispatch

transaction_status_changed = django.dispatch.Signal(providing_args=["instance", "old_status"])
"""Signal that is fired after transaction status is changed"""

transaction_failure_send = django.dispatch.Signal(providing_args=["instance"])
"""Send signal when transaction is fail"""



# -*- coding: utf-8 -*-
import json

from django.conf import settings
from django.test import TestCase

from requests import Response

from mock import patch

import ripple_api

data = {
    u'status': u'success',
    u'lines': [
        {
            u'account': u'rhhPzptf4EdiRRopSVC3AEbHwDa9df8y2i',
            u'quality_out': 0,
            u'quality_in': 0,
            u'currency': u'USD',
            u'limit_peer': u'5',
            u'limit': u'2.5',
            u'balance': u'1.759565201742073',
        }, {
            u'account': u'rhhPzptf4EdiRRopSVC3AEbHwDa9df8y2i',
            u'quality_out': 0,
            u'no_ripple_peer': True,
            u'quality_in': 0,
            u'currency': u'CCK',
            u'limit_peer': u'2',
            u'limit': u'0',
            u'balance': u'0',
        }, {
            u'account': u'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7',
            u'quality_out': 0,
            u'quality_in': 0,
            u'currency': u'CCK',
            u'limit_peer': u'0',
            u'limit': u'1000000',
            u'balance': u'7.51418646934461',
        }, {
            u'account': u'rp2PaYDxVwDvaZVLEQv7bHhoFQEyX1mEx7',
            u'quality_out': 0,
            u'quality_in': 0,
            u'currency': u'USD',
            u'limit_peer': u'0',
            u'limit': u'10000',
            u'balance': u'0.7907',
        }],
    u'account': u'rJmEYVEB7Bw16Kt9FYHK74Xrgery6gN6AB',
    u'validated': False,
    u'ledger_current_index': 12159817,
}

untrusted_data = {
    "account": "rJmEYVEB7Bw16Kt9FYHK74Xrgery6gN6AB",
    "lines": [],
    "status": "success"
}


class IsTrustSetTestCase(TestCase):

    def setUp(self):
        self.servers = settings.RIPPLE_API_DATA
        self.account = u'rJmEYVEB7Bw16Kt9FYHK74Xrgery6gN6AB'
        self.trusted_peer = u"rhhPzptf4EdiRRopSVC3AEbHwDa9df8y2i"
        self.untrusted_peer = u"rJobmmpNqozqY7MzwGkRs1VLEBJ7H5Pjrp"
        self.usd_granted_limit = 1
        self.usd_overgranted_limit = 10

    @patch('requests.post')
    def test_detect_trusted_peer(self, post_mock):
        """ Test if is_trust_set detects trusted peer
        """
        response = Response()
        response._content = json.dumps({u"result": data})
        post_mock.return_value = response

        is_trusted = ripple_api.is_trust_set(self.account,
                                             self.trusted_peer,
                                             currency="USD",
                                             servers=self.servers)

        self.assertEqual(is_trusted, True)

    @patch('requests.post')
    def test_detect_untrusted_peer(self, post_mock):
        """ Test if is_trust_set detects untrusted peer
        """
        response = Response()
        response._content = json.dumps({u"result": untrusted_data})
        post_mock.return_value = response

        is_trusted = ripple_api.is_trust_set(self.account,
                                             self.untrusted_peer,
                                             currency="USD",
                                             servers=self.servers)

        self.assertEqual(is_trusted, False)

    @patch('requests.post')
    def test_detect_trusted_in_general_not_trusted_in_eur(self, post_mock):
        """ Test if is_trust_set detects trusted in general peer
            but not trusted in certain currency (EUR)
        """
        response = Response()
        response._content = json.dumps({u"result": data})
        post_mock.return_value = response

        is_trusted_in_general = ripple_api.is_trust_set(self.account,
                                                        self.trusted_peer,
                                                        servers=self.servers)

        self.assertEqual(is_trusted_in_general, True)
        is_trusted_eur = ripple_api.is_trust_set(self.account,
                                                 self.trusted_peer,
                                                 currency="EUR",
                                                 servers=self.servers)

        self.assertEqual(is_trusted_eur, False)

    @patch('requests.post')
    def test_detect_limit_enough(self, post_mock):
        """ Test if is_trust_set detects if limit is enough
        """
        response = Response()
        response._content = json.dumps({u"result": data})
        post_mock.return_value = response

        is_trusted = ripple_api.is_trust_set(self.account,
                                             self.untrusted_peer,
                                             currency="USD",
                                             limit=self.usd_granted_limit,
                                             servers=self.servers)

        self.assertEqual(is_trusted, True)

    @patch('requests.post')
    def test_detect_limit_not_enough(self, post_mock):
        """ Test if is_trust_set detects if limit is not enough
        """
        response = Response()
        response._content = json.dumps({u"result": data})
        post_mock.return_value = response

        is_trusted = ripple_api.is_trust_set(self.account,
                                             self.untrusted_peer,
                                             currency="USD",
                                             limit=self.usd_overgranted_limit,
                                             servers=self.servers)

        self.assertEqual(is_trusted, False)



# -*- coding: utf-8 -*-
from requests import ConnectionError
from celery import task
import logging
import time

from .models import Transaction
from ripple_api import RippleApiError, path_find, sign, submit, tx


@task
def sign_task(transaction_pk, secret):
    logger = logging.getLogger('ripple')

    transaction = Transaction.objects.filter(pk=transaction_pk)
    if not transaction.exists():
        logger.error("sign_task: transaction %s not found in DB!" % transaction_pk)
        return
    transaction = transaction[0]
    try:
        if transaction.currency == 'XRP':
            amount = transaction.value
        else:
            amount = {"currency": transaction.currency, 
                      "value": str(transaction.value), 
                      "issuer": transaction.destination}
            paths = path_find(transaction.account,
                              transaction.destination,
                              amount,
                              [{'currency': transaction.currency,
                                'issuer': transaction.account,
                                },
                               ])
            if len(paths['alternatives'])==0:
                raise RippleApiError('no path', '', 
                                     'No path between %s and %s for %s' % (
                        transaction.account, transaction.destination, amount))
            pass

        response = sign(transaction.account, 
                        secret, 
                        transaction.destination, 
                        amount,
                        paths = paths['alternatives'][0]['paths_computed']
                        )
    except (RippleApiError, ConnectionError), e:
        transaction.status = Transaction.FAILURE
        transaction.save()
        logger.error(e)
        return

    transaction.hash = response['tx_json']['hash']
    transaction.tx_blob = response['tx_blob']
    transaction.status = Transaction.PENDING
    transaction.save()

    logger.info('Transaction signed: %s' % transaction)
    return transaction.pk


@task
def submit_task(transaction_pk):
    if not transaction_pk:
        return
    transaction = Transaction.objects.filter(pk=transaction_pk)
    logger = logging.getLogger('ripple')
    if not transaction.exists():
        logger.error("sign_task: transaction %s not found in DB!" % transaction_pk)
        return
    transaction = transaction[0]
    try:
        response = submit(transaction.tx_blob)
        if response['engine_result'] == "telINSUF_FEE_P":
            # ripple server too busy to forward or process your transaction
            while True:
                time.sleep(2)
                tx_response = tx(transaction.hash)
                status = tx_response.get('meta', {}).get('TransactionResult')
                # check if tx status is available
                if status:
                    response['engine_result'] = status
                    break

    except RippleApiError, e:
        logger.error(e)
        transaction.status = Transaction.FAILURE
        transaction.save()
        return
    except ConnectionError, e:
        logger.error('Connection error: %s' % e)
        return

    if response['engine_result'] in ["tesSUCCESS",  "tefPAST_SEQ"]:
        transaction.status = Transaction.SUBMITTED
        transaction.save()
        logger.info("Transaction: %s successful submitted." % transaction)
    else:
        transaction.status = Transaction.FAILURE
        transaction.save()
        logger.info("Transaction: %s submitted with result %s" % (transaction, response['engine_result']))
